package tudu.web.dwr.impl;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Set;
import java.util.TreeSet;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.DataAccessException;

import tudu.domain.model.Todo;
import tudu.domain.model.TodoList;
import tudu.domain.model.User;
import tudu.security.PermissionDeniedException;
import tudu.service.TodoListsManager;
import tudu.service.TodosManager;
import tudu.service.UserManager;
import tudu.web.dwr.TodosDwr;
import tudu.web.dwr.bean.RemoteTodo;
import tudu.web.dwr.bean.RemoteTodoList;
import uk.ltd.getahead.dwr.ExecutionContext;

/**
 * Implementation of the tudu.service.TodosManager interface.
 * 
 * @author Julien Dubois
 */
public class TodosDwrImpl extends AbstractCommonDwr implements TodosDwr {
    
    private final Log log = LogFactory.getLog(TodosDwrImpl.class);
    
    private UserManager userManager = null;
    
    private TodosManager todosManager = null;
    
    private TodoListsManager todoListsManager = null;

    public void setUserManager(UserManager userManager) {
        this.userManager = userManager;
    }

    public void setTodosManager(TodosManager todosManager) {
        this.todosManager = todosManager;
    }

    public void setTodoListsManager(TodoListsManager todoListsManager) {
        this.todoListsManager = todoListsManager;
    }

    /**
     * @see tudu.web.dwr.TodosDwr#getCurrentTodoLists()
     */
    public RemoteTodoList[] getCurrentTodoLists() {
        log.debug("Get the current Todo Lists (AJAX menu generation)");
        User user = userManager.getCurrentUser();
        Collection<TodoList> todoLists = user.getTodoLists();
        Collection<RemoteTodoList> remoteTodoLists = 
            new TreeSet<RemoteTodoList>();
        
        for (TodoList todoList : todoLists) {
            RemoteTodoList remoteTodoList = new RemoteTodoList();
            remoteTodoList.setListId(todoList.getListId());
            remoteTodoList.setName(todoList.getName());
            int completed = 0;
            for (Todo todo : todoList.getTodos()) {
                if (todo.isCompleted()) {
                    completed++;
                }
            }
            remoteTodoList.setDescription(todoList.getName() 
                    + " (" + completed + "/"
                    + todoList.getTodos().size() + ")");
            
            remoteTodoLists.add(remoteTodoList);
        }
        return (RemoteTodoList[]) 
            remoteTodoLists.toArray(new RemoteTodoList[remoteTodoLists.size()]);
    }

    /**
     * @see tudu.web.dwr.TodosDwr#getTodoById(java.lang.String)
     */
    public RemoteTodo getTodoById(String todoId) {
        Todo todo = todosManager.findTodo(todoId);
        RemoteTodo remoteTodo = new RemoteTodo();
        remoteTodo.setDescription(todo.getDescription());
        remoteTodo.setPriority(todo.getPriority());
        if (todo.getDueDate() != null) {
            SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
            String formattedDate = formatter.format(todo.getDueDate());
            remoteTodo.setDueDate(formattedDate);
        } else {
            remoteTodo.setDueDate("");
        }
        return remoteTodo;
    }

    /**
     * @see tudu.web.dwr.TodosDwr#renderTodos(java.lang.String)
     */
    public String renderTodos(String listId) {
        log.debug("Render AJAX table");
        HttpServletRequest request = ExecutionContext.get()
                .getHttpServletRequest();
        if (listId != null && !listId.equals("")) {
            try {
                TodoList todoList = todoListsManager.findTodoList(listId);
                Set<Todo> todos = todoList.getTodos();
                request.setAttribute("todoList", todoList);
                request.setAttribute("todos", new TreeSet<Todo>(todos));
                int nbCompleted = 0;
                for (Todo todo : todos) {
                    if (todo.isCompleted()) {
                        nbCompleted++;
                    }
                }
                if (todos.size() != 0) {
                    request.setAttribute("completion", nbCompleted * 100
                            / todos.size());
                } else {
                    request.setAttribute("completion", 100);
                }
            } catch (DataAccessException dae) {
                request.setAttribute("todos", new ArrayList());
            }
        } else {
            request.setAttribute("todos", new ArrayList());
        }
        try {
            return ExecutionContext.get().forwardToString(
                    "/WEB-INF/jspf/todos_table.jsp");
        } catch (ServletException e) {
            log.error("ServletException : " + e);
            return "";
        } catch (IOException ioe) {
            log.error("IOException : " + ioe);
            return "";
        }
    }

    /**
     * @see tudu.web.dwr.TodosDwr#addTodo(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public String addTodo(String listId, String description, String priority, String dueDate) {
        log.debug("Execute addTodo action");
        Todo todo = new Todo();
        todo.setDescription(description);
        
        int priorityInt = 0;
        try {
            priorityInt = Integer.valueOf(priority);
        } catch (NumberFormatException e) {
            // The priority is not a number.
        }
        todo.setPriority(priorityInt);
        
        SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
        try {
            Date due = formatter.parse(dueDate);
            todo.setDueDate(due);
        } catch (ParseException e) {
            // The date is not correct
        }
        
        try {
            todosManager.createTodo(listId, todo);
            return renderTodos(listId);
        } catch (PermissionDeniedException pde) {
            return renderPermissionException();
        }
    }

    /**
     * @see tudu.web.dwr.TodosDwr#reopenTodo(java.lang.String)
     */
    public String reopenTodo(String todoId) {
        log.debug("Execute reopenTodo action");
        try {
            Todo todo = todosManager.reopenTodo(todoId);
            return renderTodos(todo.getTodoList().getListId());
        } catch (PermissionDeniedException pde) {
            return renderPermissionException();
        }
    }

    /**
     * @see tudu.web.dwr.TodosDwr#completeTodo(java.lang.String)
     */
    public String completeTodo(String todoId) {
        log.debug("Execute completeTodo action");
        try {
            Todo todo = todosManager.completeTodo(todoId);
            return renderTodos(todo.getTodoList().getListId());
        } catch (PermissionDeniedException pde) {
            return renderPermissionException();
        }
    }

    /**
     * @see tudu.web.dwr.TodosDwr#editTodo(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public String editTodo(String todoId, String description, String priority, String dueDate) {
        log.debug("Execute editTodo action");
        try {
            Todo todo = todosManager.findTodo(todoId);
            todo.setDescription(description);
            
            int priorityInt = 0;
            try {
                priorityInt = Integer.parseInt(priority);
            } catch (NumberFormatException e) {
                // The priority is not a number.
            }
            todo.setPriority(priorityInt);
            
            SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
            try {
                Date due = formatter.parse(dueDate);
                todo.setDueDate(due);
            } catch (ParseException e) {
                // The date is not correct
            }
            todosManager.updateTodo(todo );
            return renderTodos(todo.getTodoList().getListId());
        } catch (PermissionDeniedException pde) {
            return renderPermissionException();
        }
    }

    /**
     * @see tudu.web.dwr.TodosDwr#deleteTodo(java.lang.String)
     */
    public String deleteTodo(String todoId) {
        Todo todo = todosManager.findTodo(todoId);
        try {
            String listId = todo.getTodoList().getListId();
            todosManager.deleteTodo(todoId);
            return renderTodos(listId);
        } catch (PermissionDeniedException pde) {
            return renderPermissionException();
        }
    }
}
